*
* avet.F
*
* Jonathan Callahan
* Feb 19th 1998
*
* Returns the argument averaged over the T axis
*


*
* In this subroutine we provide information about
* the function.  The user configurable information 
* consists of the following:
*
* descr              Text description of the function
*
* num_args           Required number of arguments
*
* axis_inheritance   Type of axis for the result
*                       ( CUSTOM, IMPLIED_BY_ARGS, NORMAL, ABSTRACT )
*                       CUSTOM          - user defined axis
*                       IMPLIED_BY_ARGS - same axis as the incoming argument
*                       NORMAL          - the result is normal to this axis
*                       ABSTRACT        - an axis which only has index values
*
* piecemeal_ok       For memory optimization:
*                       axes where calculation may be performed piecemeal
*                       ( YES, NO )
* 
*
* For each argument we provide the following information:
*
* name               Text name for an argument
*
* unit               Text units for an argument
*
* desc               Text description of an argument
*
* axis_influence     Are this argument's axes the same as the result grid?
*                       ( YES, NO )
*
* axis_extend       How much does Ferret need to extend arg limits relative to result 
*


      SUBROUTINE avet_init(id)

      INCLUDE 'ferret_cmn/EF_Util.cmn'

      INTEGER id, arg

***********************************************************************
*                                           USER CONFIGURABLE PORTION |
*                                                                     |
*                                                                     V

      CALL efn_set_desc(id,'returns the time average' )

      CALL efn_set_num_args(id, 1)
      CALL efn_set_has_vari_args(id, NO)
      CALL efn_set_axis_inheritance(id, IMPLIED_BY_ARGS, 
     .     IMPLIED_BY_ARGS, IMPLIED_BY_ARGS, NORMAL)
      CALL efn_set_piecemeal_ok(id, YES, YES, YES, NO)

      arg = 1
      CALL efn_set_axis_influence(id, arg, YES, YES, YES, YES)
*                                                                     ^
*                                                                     |
*                                           USER CONFIGURABLE PORTION |
***********************************************************************

      RETURN 
      END


*
* In this subroutine we compute the result
*
      SUBROUTINE avet_compute(id, arg_1, result)

      INCLUDE 'ferret_cmn/EF_Util.cmn'
      INCLUDE 'ferret_cmn/xmem_subsc.cmn'

      INTEGER id

      REAL bad_flag(EF_MAX_ARGS), bad_flag_result
      REAL arg_1(m1lox:m1hix, m1loy:m1hiy, m1loz:m1hiz, m1lot:m1hit)
      REAL result(m10lox:m10hix, m10loy:m10hiy, m10loz:m10hiz, 
     .     m10lot:m10hit)

* After initialization, the 'res_' arrays contain indexing information 
* for the result axes.  The 'arg_' arrays will contain the indexing 
* information for each variable's axes. 

      INTEGER res_lo_ss(4), res_hi_ss(4), res_incr(4)
      INTEGER arg_lo_ss(4,EF_MAX_ARGS), arg_hi_ss(4,EF_MAX_ARGS),
     .     arg_incr(4,EF_MAX_ARGS)


***********************************************************************
*                                           USER CONFIGURABLE PORTION |
*                                                                     |
*                                                                     V

      INTEGER i, j, k, l
      INTEGER i1, j1, k1, l1
      REAL    denominator
      REAL    sum
      LOGICAL data_exists

      CALL efn_get_res_subscripts(id, res_lo_ss, res_hi_ss, res_incr)
      CALL efn_get_arg_subscripts(id, arg_lo_ss, arg_hi_ss, arg_incr)
      CALL efn_get_bad_flags(id, bad_flag, bad_flag_result)

      i1 = arg_lo_ss(X_AXIS,ARG1)
      DO 400 i=res_lo_ss(X_AXIS), res_hi_ss(X_AXIS)

         j1 = arg_lo_ss(Y_AXIS,ARG1)
         DO 300 j=res_lo_ss(Y_AXIS), res_hi_ss(Y_AXIS)

            k1 = arg_lo_ss(Z_AXIS,ARG1)
            DO 200 k=res_lo_ss(Z_AXIS), res_hi_ss(Z_AXIS)

*
* Loop through time and calculate a running sum.
*
               sum = 0
               denominator = 0
               data_exists = .FALSE.

               l = res_lo_ss(T_AXIS)
               DO 100 l1=arg_lo_ss(T_AXIS,ARG1), arg_hi_ss(T_AXIS,ARG1)

                  IF ( arg_1(i1,j1,k1,l1) .EQ. bad_flag(ARG1) ) THEN

                     result(i,j,k,l) = bad_flag_result

                  ELSE

                     data_exists = .TRUE.
                     denominator = denominator + 1.0
                     sum = sum + arg_1(i1,j1,k1,l1)

                  END IF

 100           CONTINUE

*
* If we had any good data, calculate the average.
*
               IF ( data_exists ) THEN
                  result(i,j,k,l) = sum/denominator
               END IF

               k1 = k1 + arg_incr(Z_AXIS,ARG1)
 200        CONTINUE

            j1 = j1 + arg_incr(Y_AXIS,ARG1)
 300     CONTINUE

         i1 = i1 + arg_incr(X_AXIS,ARG1)
 400  CONTINUE
      
*                                                                     ^
*                                                                     |
*                                           USER CONFIGURABLE PORTION |
***********************************************************************

      RETURN 
      END
